@@ -15,8 +15,11 @@
  * can be freed in strict order.
  */
 struct qc_stream_buf {
+	struct eb_root ack_tree; /* storage for out-of-order ACKs */
+	struct eb64_node offset_node; /* node for qc_stream_desc buf tree */
 	struct buffer buf; /* STREAM payload */
-	struct list list; /* element for qc_stream_desc list */
+	uint64_t room; /* room already notified from buffered ACKs */
+	int sbuf;
 };
 
 #define QC_SD_FL_RELEASE	0x00000001 /* set when MUX has finished to use this stream */
@@ -35,16 +38,24 @@ struct qc_stream_desc {
 	struct eb64_node by_id; /* node for quic_conn tree */
 	struct quic_conn *qc;
 
-	struct list buf_list; /* buffers waiting for ACK, oldest offset first */
 	struct qc_stream_buf *buf; /* current buffer used by the MUX */
 	uint64_t buf_offset; /* base offset of current buffer */
 
 	uint64_t ack_offset; /* last acknowledged offset */
-	struct eb_root acked_frms; /* ACK frames tree for non-contiguous ACK ranges */
+	struct eb_root buf_tree; /* list of active and released buffers */
 
 	int flags; /* QC_SD_FL_* values */
 
-	void *ctx; /* MUX specific context */
+	void (*notify_send)(struct qc_stream_desc *, uint64_t offset, uint64_t len);
+	void (*notify_room)(struct qc_stream_desc *, uint64_t room);
+	void *ctx; /* notify context */
+};
+
+/* Represents a range of acknowledged data that cannot be immediately deleted. */
+struct qc_stream_ack {
+	struct eb64_node offset_node; /* range starting offset, used as attach point to streambuf <ack_tree>. */
+	uint64_t len;                 /* length of the acknowledged range */
+	int fin;                      /* set if the related STREAM frame had FIN bit set */
 };
 
 #endif /* USE_QUIC */
