@@ -60,12 +60,14 @@
 #define TASK_F_USR1       0x00010000  /* preserved user flag 1, application-specific, def:0 */
 #define TASK_F_UEVT1      0x00020000  /* one-shot user event type 1, application specific, def:0 */
 #define TASK_F_UEVT2      0x00040000  /* one-shot user event type 2, application specific, def:0 */
+#define TASK_F_WANTS_TIME 0x00080000  /* task/tasklet wants th_ctx->sched_call_date to be set */
 #define TASK_F_UEVT3      0x00100000  /* one-shot user event type 3, application specific, def:0 */
 /* unused: 0x200000..0x80000000 */
 
 /* These flags are persistent across scheduler calls */
 #define TASK_PERSISTENT   (TASK_SELF_WAKING | TASK_KILLED | \
-                           TASK_HEAVY | TASK_F_TASKLET | TASK_F_USR1)
+                           TASK_HEAVY | TASK_F_TASKLET | TASK_F_USR1 | \
+                           TASK_F_WANTS_TIME)
 
 /* This function is used to report state in debugging tools. Please reflect
  * below any single-bit flag addition above in the same order via the
