@@ -819,7 +819,12 @@ struct task *task_run_applet(struct task *t, void *context, unsigned int state)
 
 	input  = ic->total;
 	output = co_data(oc);
-	app->applet->fct(app);
+
+	/* Don't call I/O handler if the applet was shut (release callback was
+	 * already called)
+	 */
+	if (!se_fl_test(app->sedesc, SE_FL_SHR | SE_FL_SHW))
+		app->applet->fct(app);
 
 	TRACE_POINT(APPLET_EV_PROCESS, app);
 
@@ -916,7 +921,11 @@ struct task *task_process_applet(struct task *t, void *context, unsigned int sta
 	applet_need_more_data(app);
 	applet_have_no_more_data(app);
 
-	app->applet->fct(app);
+	/* Don't call I/O handler if the applet was shut (release callback was
+	 * already called)
+	 */
+	if (!applet_fl_test(app, APPCTX_FL_SHUTDOWN))
+		app->applet->fct(app);
 
 	TRACE_POINT(APPLET_EV_PROCESS, app);
 
@@ -949,7 +958,7 @@ struct task *task_process_applet(struct task *t, void *context, unsigned int sta
 	}
 
 	/* measure the call rate and check for anomalies when too high */
-	if ((!did_recv && sc_is_send_allowed(sc)) && (!did_send && sc_is_recv_allowed(sc))) {
+	if ((!did_send && sc_is_send_allowed(sc)) && (!did_recv && sc_is_recv_allowed(sc))) {
 		rate = update_freq_ctr(&app->call_rate, 1);
 		if (rate >= 100000 && app->call_rate.prev_ctr) // looped like this more than 100k times over last second
 			stream_dump_and_crash(&app->obj_type, read_freq_ctr(&app->call_rate));
